@@ -18,9 +18,10 @@ use strict;
 use warnings;
 use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
+use Excel::Writer::XLSX::Worksheet;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '0.46';
+our $VERSION = '0.48';
 
 
 ###############################################################################
@@ -98,7 +99,7 @@ sub _assemble_xml_file {
 #
 # _add_drawing_object()
 #
-# Add a chart or image sub object to the drawing.
+# Add a chart, image or shape sub object to the drawing.
 #
 sub _add_drawing_object {
 
@@ -168,7 +169,9 @@ sub _write_two_cell_anchor {
     my $width           = shift;
     my $height          = shift;
     my $description     = shift;
-    my @attributes      = ();
+    my $shape           = shift;
+
+    my @attributes = ();
 
 
     # Add attribute for images.
@@ -176,6 +179,9 @@ sub _write_two_cell_anchor {
         push @attributes, ( editAs => 'oneCell' );
     }
 
+    # Add editAs attribute for shapes.
+    push @attributes, ( editAs => $shape->{_editAs} ) if $shape->{_editAs};
+
     $self->{_writer}->startTag( 'xdr:twoCellAnchor', @attributes );
 
     # Write the xdr:from element.
@@ -198,15 +204,23 @@ sub _write_two_cell_anchor {
 
     if ( $type == 1 ) {
 
+        # Graphic frame.
+
         # Write the xdr:graphicFrame element for charts.
-        $self->_write_graphic_frame( $index );
+        $self->_write_graphic_frame( $index, $description );
     }
-    else {
+    elsif ( $type == 2 ) {
 
         # Write the xdr:pic element.
         $self->_write_pic( $index, $col_absolute, $row_absolute, $width,
             $height, $description );
     }
+    else {
+
+        # Write the xdr:sp element for shapes.
+        $self->_write_sp( $index, $col_absolute, $row_absolute, $width, $height,
+            $shape );
+    }
 
     # Write the xdr:clientData element.
     $self->_write_client_data();
@@ -433,16 +447,17 @@ sub _write_ext {
 #
 sub _write_graphic_frame {
 
-    my $self   = shift;
-    my $index  = shift;
-    my $macro  = '';
+    my $self  = shift;
+    my $index = shift;
+    my $name  = shift;
+    my $macro = '';
 
     my @attributes = ( 'macro' => $macro );
 
     $self->{_writer}->startTag( 'xdr:graphicFrame', @attributes );
 
     # Write the xdr:nvGraphicFramePr element.
-    $self->_write_nv_graphic_frame_pr( $index );
+    $self->_write_nv_graphic_frame_pr( $index, $name );
 
     # Write the xdr:xfrm element.
     $self->_write_xfrm();
@@ -463,12 +478,17 @@ sub _write_graphic_frame {
 sub _write_nv_graphic_frame_pr {
 
     my $self  = shift;
-    my $index  = shift;
+    my $index = shift;
+    my $name  = shift;
+
+    if ( !$name ) {
+        $name = 'Chart ' . $index;
+    }
 
     $self->{_writer}->startTag( 'xdr:nvGraphicFramePr' );
 
     # Write the xdr:cNvPr element.
-    $self->_write_c_nv_pr( $index + 1, 'Chart ' . $index );
+    $self->_write_c_nv_pr( $index + 1, $name );
 
     # Write the xdr:cNvGraphicFramePr element.
     $self->_write_c_nv_graphic_frame_pr();
@@ -691,6 +711,127 @@ sub _write_client_data {
 }
 
 
+##############################################################################
+#
+# _write_sp()
+#
+# Write the <xdr:sp> element.
+#
+sub _write_sp {
+
+    my $self         = shift;
+    my $index        = shift;
+    my $col_absolute = shift;
+    my $row_absolute = shift;
+    my $width        = shift;
+    my $height       = shift;
+    my $shape        = shift;
+
+    if ( $shape->{_connect} ) {
+        my @attributes = ( macro => '' );
+        $self->{_writer}->startTag( 'xdr:cxnSp', @attributes );
+
+        # Write the xdr:nvCxnSpPr element.
+        $self->_write_nv_cxn_sp_pr( $index, $shape );
+
+        # Write the xdr:spPr element.
+        $self->_write_xdr_sp_pr( $index, $col_absolute, $row_absolute, $width,
+            $height, $shape );
+
+        $self->{_writer}->endTag( 'xdr:cxnSp' );
+    }
+    else {
+
+        # Add attribute for shapes.
+        my @attributes = ( macro => '', textlink => '' );
+        $self->{_writer}->startTag( 'xdr:sp', @attributes );
+
+        # Write the xdr:nvSpPr element.
+        $self->_write_nv_sp_pr( $index, $shape );
+
+        # Write the xdr:spPr element.
+        $self->_write_xdr_sp_pr( $index, $col_absolute, $row_absolute, $width,
+            $height, $shape );
+
+        # Write the xdr:txBody element.
+        if ( $shape->{_text} ) {
+            $self->_write_txBody( $col_absolute, $row_absolute, $width, $height,
+                $shape );
+        }
+
+        $self->{_writer}->endTag( 'xdr:sp' );
+    }
+}
+##############################################################################
+#
+# _write_nv_cxn_sp_pr()
+#
+# Write the <xdr:nvCxnSpPr> element.
+#
+sub _write_nv_cxn_sp_pr {
+
+    my $self  = shift;
+    my $index = shift;
+    my $shape = shift;
+
+    $self->{_writer}->startTag( 'xdr:nvCxnSpPr' );
+
+    $shape->{_name} = join( ' ', $shape->{_type}, $index )
+      unless defined $shape->{_name};
+    $self->_write_c_nv_pr( $shape->{_id}, $shape->{_name} );
+
+    $self->{_writer}->startTag( 'xdr:cNvCxnSpPr' );
+
+    my @attributes = ( noChangeShapeType => '1' );
+    $self->{_writer}->emptyTag( 'a:cxnSpLocks', @attributes );
+
+    if ( $shape->{_start} ) {
+        @attributes = ( 'id' => $shape->{_start}, 'idx' => $shape->{_start_index} );
+        $self->{_writer}->emptyTag( 'a:stCxn', @attributes );
+    }
+
+    if ( $shape->{_end} ) {
+        @attributes = ( 'id' => $shape->{_end}, 'idx' => $shape->{_end_index} );
+        $self->{_writer}->emptyTag( 'a:endCxn', @attributes );
+    }
+    $self->{_writer}->endTag( 'xdr:cNvCxnSpPr' );
+    $self->{_writer}->endTag( 'xdr:nvCxnSpPr' );
+}
+
+
+##############################################################################
+#
+# _write_nv_sp_pr()
+#
+# Write the <xdr:NvSpPr> element.
+#
+sub _write_nv_sp_pr {
+
+    my $self  = shift;
+    my $index = shift;
+    my $shape = shift;
+
+    my @attributes = ();
+
+    $self->{_writer}->startTag( 'xdr:nvSpPr' );
+
+    my $shape_name = $shape->{_type} . ' ' . $index;
+
+    $self->_write_c_nv_pr( $shape->{_id}, $shape_name );
+
+    @attributes = ( 'txBox' => 1 ) if $shape->{_txBox};
+
+    $self->{_writer}->startTag( 'xdr:cNvSpPr', @attributes );
+
+    @attributes = ( noChangeArrowheads => '1' );
+
+    $self->{_writer}->emptyTag( 'a:spLocks', @attributes );
+
+    $self->{_writer}->endTag( 'xdr:cNvSpPr' );
+    $self->{_writer}->endTag( 'xdr:nvSpPr' );
+}
+
+
 ##############################################################################
 #
 # _write_pic()
@@ -715,8 +856,12 @@ sub _write_pic {
     # Write the xdr:blipFill element.
     $self->_write_blip_fill( $index );
 
+    # Pictures are rectangle shapes by default.
+    my $shape = { _type => 'rect' };
+
     # Write the xdr:spPr element.
-    $self->_write_sp_pr( $col_absolute, $row_absolute, $width, $height );
+    $self->_write_sp_pr( $col_absolute, $row_absolute, $width, $height,
+        $shape );
 
     $self->{_writer}->endTag( 'xdr:pic' );
 }
@@ -754,7 +899,7 @@ sub _write_nv_pic_pr {
 #
 sub _write_c_nv_pic_pr {
 
-    my $self                 = shift;
+    my $self = shift;
 
     $self->{_writer}->startTag( 'xdr:cNvPicPr' );
 
@@ -865,7 +1010,7 @@ sub _write_a_fill_rect {
 #
 # _write_sp_pr()
 #
-# Write the <xdr:spPr> element.
+# Write the <xdr:spPr> element, for charts.
 #
 sub _write_sp_pr {
 
@@ -874,6 +1019,7 @@ sub _write_sp_pr {
     my $row_absolute = shift;
     my $width        = shift;
     my $height       = shift;
+    my $shape        = shift || {};
 
     $self->{_writer}->startTag( 'xdr:spPr' );
 
@@ -881,12 +1027,56 @@ sub _write_sp_pr {
     $self->_write_a_xfrm( $col_absolute, $row_absolute, $width, $height );
 
     # Write the a:prstGeom element.
-    $self->_write_a_prst_geom();
+    $self->_write_a_prst_geom( $shape );
 
     $self->{_writer}->endTag( 'xdr:spPr' );
 }
 
 
+##############################################################################
+#
+# _write_xdr_sp_pr()
+#
+# Write the <xdr:spPr> element for shapes.
+#
+sub _write_xdr_sp_pr {
+
+    my $self         = shift;
+    my $index        = shift;
+    my $col_absolute = shift;
+    my $row_absolute = shift;
+    my $width        = shift;
+    my $height       = shift;
+    my $shape        = shift;
+
+    my @attributes = ( 'bwMode' => 'auto' );
+
+    $self->{_writer}->startTag( 'xdr:spPr', @attributes );
+
+    # Write the a:xfrm element.
+    $self->_write_a_xfrm( $col_absolute, $row_absolute, $width, $height,
+        $shape );
+
+    # Write the a:prstGeom element.
+    $self->_write_a_prst_geom( $shape );
+
+    my $fill = $shape->{_fill};
+
+    if ( length $fill > 1 ) {
+
+        # Write the a:solidFill element.
+        $self->_write_a_solid_fill( $fill );
+    }
+    else {
+        $self->{_writer}->emptyTag( 'a:noFill' );
+    }
+
+    # Write the a:ln element.
+    $self->_write_a_ln( $shape );
+
+    $self->{_writer}->endTag( 'xdr:spPr' );
+}
+
 ##############################################################################
 #
 # _write_a_xfrm()
@@ -900,8 +1090,17 @@ sub _write_a_xfrm {
     my $row_absolute = shift;
     my $width        = shift;
     my $height       = shift;
+    my $shape        = shift || {};
+    my @attributes   = ();
 
-    $self->{_writer}->startTag( 'a:xfrm' );
+    my $rotation = $shape->{_rotation} || 0;
+    $rotation *= 60000;
+
+    push( @attributes, ( 'rot'   => $rotation ) ) if $rotation;
+    push( @attributes, ( 'flipH' => 1 ) )         if $shape->{_flip_h};
+    push( @attributes, ( 'flipV' => 1 ) )         if $shape->{_flip_v};
+
+    $self->{_writer}->startTag( 'a:xfrm', @attributes );
 
     # Write the a:off element.
     $self->_write_a_off( $col_absolute, $row_absolute );
@@ -964,14 +1163,16 @@ sub _write_a_ext {
 sub _write_a_prst_geom {
 
     my $self = shift;
-    my $prst = 'rect';
+    my $shape = shift || {};
 
-    my @attributes = ( 'prst' => $prst );
+    my @attributes = ();
+
+    @attributes = ( 'prst' => $shape->{_type} ) if $shape->{_type};
 
     $self->{_writer}->startTag( 'a:prstGeom', @attributes );
 
     # Write the a:avLst element.
-    $self->_write_a_av_lst();
+    $self->_write_a_av_lst( $shape );
 
     $self->{_writer}->endTag( 'a:prstGeom' );
 }
@@ -985,15 +1186,216 @@ sub _write_a_prst_geom {
 #
 sub _write_a_av_lst {
 
+    my $self        = shift;
+    my $shape       = shift || {};
+    my $adjustments = [];
+
+    if ( defined $shape->{_adjustments} ) {
+        $adjustments = $shape->{_adjustments};
+    }
+
+    if ( @$adjustments ) {
+        $self->{_writer}->startTag( 'a:avLst' );
+
+        my $i = 0;
+        foreach my $adj ( @{$adjustments} ) {
+            $i++;
+
+            # Only connectors have multiple adjustments.
+            my $suffix = $shape->{_connect} ? $i : '';
+
+            # Scale Adjustments: 100,000 = 100%.
+            my $adj_int = int( $adj * 1000 );
+
+            my @attributes =
+              ( name => 'adj' . $suffix, fmla => "val $adj_int" );
+
+            $self->{_writer}->emptyTag( 'a:gd', @attributes );
+        }
+        $self->{_writer}->endTag( 'a:avLst' );
+    }
+    else {
+        $self->{_writer}->emptyTag( 'a:avLst' );
+    }
+}
+
+
+##############################################################################
+#
+# _write_a_solid_fill()
+#
+# Write the <a:solidFill> element.
+#
+sub _write_a_solid_fill {
+
     my $self = shift;
+    my $rgb  = shift;
+
+    $rgb = '000000' unless defined $rgb;
+
+    my @attributes = ( 'val' => $rgb );
+
+    $self->{_writer}->startTag( 'a:solidFill' );
+
+    $self->{_writer}->emptyTag( 'a:srgbClr', @attributes );
+
+    $self->{_writer}->endTag( 'a:solidFill' );
+}
+
+
+##############################################################################
+#
+# _write_a_ln()
+#
+# Write the <a:ln> element.
+#
+sub _write_a_ln {
+
+    my $self = shift;
+    my $shape = shift || {};
+
+    my $weight = $shape->{_line_weight};
+
+    my @attributes = ( 'w' => $weight * 9525 );
+
+    $self->{_writer}->startTag( 'a:ln', @attributes );
+
+    my $line = $shape->{_line};
+
+    if ( length $line > 1 ) {
+
+        # Write the a:solidFill element.
+        $self->_write_a_solid_fill( $line );
+    }
+    else {
+        $self->{_writer}->emptyTag( 'a:noFill' );
+    }
+
+    if ( $shape->{_line_type} ) {
+
+        @attributes = ( 'val' => $shape->{_line_type} );
+        $self->{_writer}->emptyTag( 'a:prstDash', @attributes );
+    }
+
+    if ( $shape->{_connect} ) {
+        $self->{_writer}->emptyTag( 'a:round' );
+    }
+    else {
+        @attributes = ( 'lim' => 800000 );
+        $self->{_writer}->emptyTag( 'a:miter', @attributes );
+    }
+
+    $self->{_writer}->emptyTag( 'a:headEnd' );
+    $self->{_writer}->emptyTag( 'a:tailEnd' );
+
+    $self->{_writer}->endTag( 'a:ln' );
+}
+
+
+##############################################################################
+#
+# _write_txBody
+#
+# Write the <xdr:txBody> element.
+#
+sub _write_txBody {
+
+    my $self         = shift;
+    my $col_absolute = shift;
+    my $row_absolute = shift;
+    my $width        = shift;
+    my $height       = shift;
+    my $shape        = shift;
+
+    my @attributes = (
+        vertOverflow => "clip",
+        wrap         => "square",
+        lIns         => "27432",
+        tIns         => "22860",
+        rIns         => "27432",
+        bIns         => "22860",
+        anchor       => $shape->{_valign},
+        upright      => "1",
+    );
+
+    $self->{_writer}->startTag( 'xdr:txBody' );
+    $self->{_writer}->emptyTag( 'a:bodyPr', @attributes );
+    $self->{_writer}->emptyTag( 'a:lstStyle' );
+
+    $self->{_writer}->startTag( 'a:p' );
+
+    my $rotation = $shape->{_format}->{_rotation};
+    $rotation = 0 unless defined $rotation;
+    $rotation *= 60000;
+
+    @attributes = ( algn => $shape->{_align}, rtl => $rotation );
+    $self->{_writer}->startTag( 'a:pPr', @attributes );
+
+    @attributes = ( sz => "1000" );
+    $self->{_writer}->emptyTag( 'a:defRPr', @attributes );
+
+    $self->{_writer}->endTag( 'a:pPr' );
+    $self->{_writer}->startTag( 'a:r' );
+
+    my $size = $shape->{_format}->{_size};
+    $size = 8 unless defined $size;
+    $size *= 100;
+
+    my $bold = $shape->{_format}->{_bold};
+    $bold = 0 unless defined $bold;
+
+    my $italic = $shape->{_format}->{_italic};
+    $italic = 0 unless defined $italic;
+
+    my $underline = $shape->{_format}->{_underline};
+    $underline = $underline ? 'sng' : 'none';
+
+    my $strike = $shape->{_format}->{_font_strikeout};
+    $strike = $strike ? 'Strike' : 'noStrike';
+
+    @attributes = (
+        lang     => "en-US",
+        sz       => $size,
+        b        => $bold,
+        i        => $italic,
+        u        => $underline,
+        strike   => $strike,
+        baseline => 0,
+    );
+
+    $self->{_writer}->startTag( 'a:rPr', @attributes );
+
+    my $color = $shape->{_format}->{_color};
+    if (defined $color) {
+        $color = $shape->_get_palette_color( $color );
+        $color =~ s/^FF//;    # Remove leading FF from rgb for shape color.
+    } else {
+        $color = '000000';
+    }
+
+    $self->_write_a_solid_fill( $color );
+
+    my $font = $shape->{_format}->{_font};
+    $font = 'Calibri' unless defined $font;
+    @attributes = ( typeface => $font );
+    $self->{_writer}->emptyTag( 'a:latin', @attributes );
+
+    $self->{_writer}->emptyTag( 'a:cs', @attributes );
+
+    $self->{_writer}->endTag( 'a:rPr' );
+
+    $self->{_writer}->startTag( 'a:t' );
+    $self->{_writer}->characters( $shape->{_text} );
+    $self->{_writer}->endTag( 'a:t' );
+
+    $self->{_writer}->endTag( 'a:r' );
+    $self->{_writer}->endTag( 'a:p' );
+    $self->{_writer}->endTag( 'xdr:txBody' );
 
-    $self->{_writer}->emptyTag( 'a:avLst' );
 }
 
 
 1;
-
-
 __END__
 
 =pod
